@@ -2,6 +2,7 @@ use crate::{
     BaseConfig, HealthService, LeaseConfig, LeaseService, NodeConfig, NodeService, Register,
     RegisterConfig, Result,
 };
+use tokio::sync::oneshot::{self, Receiver};
 use tracing::info;
 
 pub async fn build_register(config: RegisterConfig) -> Result<Register> {
@@ -18,7 +19,13 @@ pub fn build_node_service(config: NodeConfig, lease_id: i64) -> NodeService {
 
 pub async fn bootstrap(
     config: BaseConfig,
-) -> Result<(Register, NodeService, HealthService, LeaseService)> {
+) -> Result<(
+    Register,
+    NodeService,
+    HealthService,
+    LeaseService,
+    Receiver<()>,
+)> {
     info!("bootstrap base service");
     // build register
     let mut register = build_register(config.register).await?;
@@ -30,9 +37,10 @@ pub async fn bootstrap(
     let lease_svc = build_lease_service(config.lease, lease_id);
     let lease_id = lease_svc.get_lease_id();
     let node_svc = build_node_service(config.node, lease_id);
-    lease_svc.run(register.clone());
-    node_svc.run(register.clone());
-    let health_svc = HealthService::default();
     // run svc
-    Ok((register, node_svc, health_svc, lease_svc))
+    lease_svc.run(register.clone());
+    let (shutdown_tx, shutdown_rx) = oneshot::channel::<()>();
+    node_svc.run(register.clone(), shutdown_tx);
+    let health_svc = HealthService::default();
+    Ok((register, node_svc, health_svc, lease_svc, shutdown_rx))
 }
