@@ -20,7 +20,7 @@ pub struct BuildResponse {
     pub version: u64,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
-pub struct CancelRequest {
+pub struct CancelBuildRequest {
     /// project namespace
     #[prost(string, tag = "1")]
     pub namespace: ::prost::alloc::string::String,
@@ -32,9 +32,9 @@ pub struct CancelRequest {
     pub build_version: u64,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
-pub struct CancelResponse {}
+pub struct CancelBuildResponse {}
 #[derive(Clone, PartialEq, ::prost::Message)]
-pub struct ScanRequest {}
+pub struct ScanBuildRequest {}
 #[derive(Clone, PartialEq, ::prost::Message)]
 pub struct BuildMetadataKey {
     /// project namespace
@@ -48,12 +48,12 @@ pub struct BuildMetadataKey {
     pub version: u64,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
-pub struct ScanResponse {
+pub struct ScanBuildResponse {
     #[prost(message, repeated, tag = "1")]
     pub builds: ::prost::alloc::vec::Vec<BuildMetadataKey>,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
-pub struct GetLogRequest {
+pub struct GetBuildLogRequest {
     /// project namespace
     #[prost(string, tag = "1")]
     pub namespace: ::prost::alloc::string::String,
@@ -65,11 +65,47 @@ pub struct GetLogRequest {
     pub build_version: u64,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
-pub struct GetLogResponse {
+pub struct GetBuildLogResponse {
     /// log context
     #[prost(bytes = "vec", tag = "1")]
     pub buffer: ::prost::alloc::vec::Vec<u8>,
 }
+#[derive(Clone, PartialEq, ::prost::Message)]
+pub struct DeleteBuildCacheRequest {
+    /// project namespace
+    #[prost(string, tag = "1")]
+    pub namespace: ::prost::alloc::string::String,
+    /// project id
+    #[prost(string, tag = "2")]
+    pub id: ::prost::alloc::string::String,
+    /// target platform
+    #[prost(string, tag = "4")]
+    pub target_platform: ::prost::alloc::string::String,
+}
+#[derive(Clone, PartialEq, ::prost::Message)]
+pub struct DeleteBuildCacheResponse {}
+#[derive(Clone, PartialEq, ::prost::Message)]
+pub struct ScanBuildCacheRequest {}
+#[derive(Clone, PartialEq, ::prost::Message)]
+pub struct BuildCacheMetadata {
+    /// project namespace
+    #[prost(string, tag = "1")]
+    pub namespace: ::prost::alloc::string::String,
+    /// project id
+    #[prost(string, tag = "2")]
+    pub id: ::prost::alloc::string::String,
+    /// target platform
+    #[prost(string, tag = "3")]
+    pub target_platform: ::prost::alloc::string::String,
+    /// cache hit timestamp
+    #[prost(message, optional, tag = "4")]
+    pub timestamp: ::core::option::Option<::prost_types::Timestamp>,
+}
+#[derive(Clone, PartialEq, ::prost::Message)]
+pub struct ScanBuildCacheResponse {
+    #[prost(message, repeated, tag = "1")]
+    pub caches: ::prost::alloc::vec::Vec<BuildCacheMetadata>,
+}
 #[doc = r" Generated client implementations."]
 pub mod builder_client {
     #![allow(unused_variables, dead_code, missing_docs, clippy::let_unit_value)]
@@ -92,7 +128,7 @@ pub mod builder_client {
     impl<T> BuilderClient<T>
     where
         T: tonic::client::GrpcService<tonic::body::BoxBody>,
-        T::ResponseBody: Body + Send + Sync + 'static,
+        T::ResponseBody: Body + Send + 'static,
         T::Error: Into<StdError>,
         <T::ResponseBody as Body>::Error: Into<StdError> + Send,
     {
@@ -144,10 +180,10 @@ pub mod builder_client {
             let path = http::uri::PathAndQuery::from_static("/build.Builder/Build");
             self.inner.unary(request.into_request(), path, codec).await
         }
-        pub async fn cancel(
+        pub async fn cancel_build(
             &mut self,
-            request: impl tonic::IntoRequest<super::CancelRequest>,
-        ) -> Result<tonic::Response<super::CancelResponse>, tonic::Status> {
+            request: impl tonic::IntoRequest<super::CancelBuildRequest>,
+        ) -> Result<tonic::Response<super::CancelBuildResponse>, tonic::Status> {
             self.inner.ready().await.map_err(|e| {
                 tonic::Status::new(
                     tonic::Code::Unknown,
@@ -155,13 +191,13 @@ pub mod builder_client {
                 )
             })?;
             let codec = tonic::codec::ProstCodec::default();
-            let path = http::uri::PathAndQuery::from_static("/build.Builder/Cancel");
+            let path = http::uri::PathAndQuery::from_static("/build.Builder/CancelBuild");
             self.inner.unary(request.into_request(), path, codec).await
         }
-        pub async fn get_log(
+        pub async fn get_build_log(
             &mut self,
-            request: impl tonic::IntoRequest<super::GetLogRequest>,
-        ) -> Result<tonic::Response<super::GetLogResponse>, tonic::Status> {
+            request: impl tonic::IntoRequest<super::GetBuildLogRequest>,
+        ) -> Result<tonic::Response<super::GetBuildLogResponse>, tonic::Status> {
             self.inner.ready().await.map_err(|e| {
                 tonic::Status::new(
                     tonic::Code::Unknown,
@@ -169,13 +205,13 @@ pub mod builder_client {
                 )
             })?;
             let codec = tonic::codec::ProstCodec::default();
-            let path = http::uri::PathAndQuery::from_static("/build.Builder/GetLog");
+            let path = http::uri::PathAndQuery::from_static("/build.Builder/GetBuildLog");
             self.inner.unary(request.into_request(), path, codec).await
         }
-        pub async fn scan(
+        pub async fn scan_build(
             &mut self,
-            request: impl tonic::IntoRequest<super::ScanRequest>,
-        ) -> Result<tonic::Response<super::ScanResponse>, tonic::Status> {
+            request: impl tonic::IntoRequest<super::ScanBuildRequest>,
+        ) -> Result<tonic::Response<super::ScanBuildResponse>, tonic::Status> {
             self.inner.ready().await.map_err(|e| {
                 tonic::Status::new(
                     tonic::Code::Unknown,
@@ -183,7 +219,35 @@ pub mod builder_client {
                 )
             })?;
             let codec = tonic::codec::ProstCodec::default();
-            let path = http::uri::PathAndQuery::from_static("/build.Builder/Scan");
+            let path = http::uri::PathAndQuery::from_static("/build.Builder/ScanBuild");
+            self.inner.unary(request.into_request(), path, codec).await
+        }
+        pub async fn delete_build_cache(
+            &mut self,
+            request: impl tonic::IntoRequest<super::DeleteBuildCacheRequest>,
+        ) -> Result<tonic::Response<super::DeleteBuildCacheResponse>, tonic::Status> {
+            self.inner.ready().await.map_err(|e| {
+                tonic::Status::new(
+                    tonic::Code::Unknown,
+                    format!("Service was not ready: {}", e.into()),
+                )
+            })?;
+            let codec = tonic::codec::ProstCodec::default();
+            let path = http::uri::PathAndQuery::from_static("/build.Builder/DeleteBuildCache");
+            self.inner.unary(request.into_request(), path, codec).await
+        }
+        pub async fn scan_build_cache(
+            &mut self,
+            request: impl tonic::IntoRequest<super::ScanBuildCacheRequest>,
+        ) -> Result<tonic::Response<super::ScanBuildCacheResponse>, tonic::Status> {
+            self.inner.ready().await.map_err(|e| {
+                tonic::Status::new(
+                    tonic::Code::Unknown,
+                    format!("Service was not ready: {}", e.into()),
+                )
+            })?;
+            let codec = tonic::codec::ProstCodec::default();
+            let path = http::uri::PathAndQuery::from_static("/build.Builder/ScanBuildCache");
             self.inner.unary(request.into_request(), path, codec).await
         }
     }
@@ -199,18 +263,26 @@ pub mod builder_server {
             &self,
             request: tonic::Request<super::BuildRequest>,
         ) -> Result<tonic::Response<super::BuildResponse>, tonic::Status>;
-        async fn cancel(
+        async fn cancel_build(
             &self,
-            request: tonic::Request<super::CancelRequest>,
-        ) -> Result<tonic::Response<super::CancelResponse>, tonic::Status>;
-        async fn get_log(
+            request: tonic::Request<super::CancelBuildRequest>,
+        ) -> Result<tonic::Response<super::CancelBuildResponse>, tonic::Status>;
+        async fn get_build_log(
             &self,
-            request: tonic::Request<super::GetLogRequest>,
-        ) -> Result<tonic::Response<super::GetLogResponse>, tonic::Status>;
-        async fn scan(
+            request: tonic::Request<super::GetBuildLogRequest>,
+        ) -> Result<tonic::Response<super::GetBuildLogResponse>, tonic::Status>;
+        async fn scan_build(
             &self,
-            request: tonic::Request<super::ScanRequest>,
-        ) -> Result<tonic::Response<super::ScanResponse>, tonic::Status>;
+            request: tonic::Request<super::ScanBuildRequest>,
+        ) -> Result<tonic::Response<super::ScanBuildResponse>, tonic::Status>;
+        async fn delete_build_cache(
+            &self,
+            request: tonic::Request<super::DeleteBuildCacheRequest>,
+        ) -> Result<tonic::Response<super::DeleteBuildCacheResponse>, tonic::Status>;
+        async fn scan_build_cache(
+            &self,
+            request: tonic::Request<super::ScanBuildCacheRequest>,
+        ) -> Result<tonic::Response<super::ScanBuildCacheResponse>, tonic::Status>;
     }
     #[derive(Debug)]
     pub struct BuilderServer<T: Builder> {
@@ -239,7 +311,7 @@ pub mod builder_server {
     impl<T, B> tonic::codegen::Service<http::Request<B>> for BuilderServer<T>
     where
         T: Builder,
-        B: Body + Send + Sync + 'static,
+        B: Body + Send + 'static,
         B::Error: Into<StdError> + Send + 'static,
     {
         type Response = http::Response<tonic::body::BoxBody>;
@@ -282,18 +354,18 @@ pub mod builder_server {
                     };
                     Box::pin(fut)
                 }
-                "/build.Builder/Cancel" => {
+                "/build.Builder/CancelBuild" => {
                     #[allow(non_camel_case_types)]
-                    struct CancelSvc<T: Builder>(pub Arc<T>);
-                    impl<T: Builder> tonic::server::UnaryService<super::CancelRequest> for CancelSvc<T> {
-                        type Response = super::CancelResponse;
+                    struct CancelBuildSvc<T: Builder>(pub Arc<T>);
+                    impl<T: Builder> tonic::server::UnaryService<super::CancelBuildRequest> for CancelBuildSvc<T> {
+                        type Response = super::CancelBuildResponse;
                         type Future = BoxFuture<tonic::Response<Self::Response>, tonic::Status>;
                         fn call(
                             &mut self,
-                            request: tonic::Request<super::CancelRequest>,
+                            request: tonic::Request<super::CancelBuildRequest>,
                         ) -> Self::Future {
                             let inner = self.0.clone();
-                            let fut = async move { (*inner).cancel(request).await };
+                            let fut = async move { (*inner).cancel_build(request).await };
                             Box::pin(fut)
                         }
                     }
@@ -302,7 +374,7 @@ pub mod builder_server {
                     let inner = self.inner.clone();
                     let fut = async move {
                         let inner = inner.0;
-                        let method = CancelSvc(inner);
+                        let method = CancelBuildSvc(inner);
                         let codec = tonic::codec::ProstCodec::default();
                         let mut grpc = tonic::server::Grpc::new(codec).apply_compression_config(
                             accept_compression_encodings,
@@ -313,18 +385,18 @@ pub mod builder_server {
                     };
                     Box::pin(fut)
                 }
-                "/build.Builder/GetLog" => {
+                "/build.Builder/GetBuildLog" => {
                     #[allow(non_camel_case_types)]
-                    struct GetLogSvc<T: Builder>(pub Arc<T>);
-                    impl<T: Builder> tonic::server::UnaryService<super::GetLogRequest> for GetLogSvc<T> {
-                        type Response = super::GetLogResponse;
+                    struct GetBuildLogSvc<T: Builder>(pub Arc<T>);
+                    impl<T: Builder> tonic::server::UnaryService<super::GetBuildLogRequest> for GetBuildLogSvc<T> {
+                        type Response = super::GetBuildLogResponse;
                         type Future = BoxFuture<tonic::Response<Self::Response>, tonic::Status>;
                         fn call(
                             &mut self,
-                            request: tonic::Request<super::GetLogRequest>,
+                            request: tonic::Request<super::GetBuildLogRequest>,
                         ) -> Self::Future {
                             let inner = self.0.clone();
-                            let fut = async move { (*inner).get_log(request).await };
+                            let fut = async move { (*inner).get_build_log(request).await };
                             Box::pin(fut)
                         }
                     }
@@ -333,7 +405,7 @@ pub mod builder_server {
                     let inner = self.inner.clone();
                     let fut = async move {
                         let inner = inner.0;
-                        let method = GetLogSvc(inner);
+                        let method = GetBuildLogSvc(inner);
                         let codec = tonic::codec::ProstCodec::default();
                         let mut grpc = tonic::server::Grpc::new(codec).apply_compression_config(
                             accept_compression_encodings,
@@ -344,18 +416,18 @@ pub mod builder_server {
                     };
                     Box::pin(fut)
                 }
-                "/build.Builder/Scan" => {
+                "/build.Builder/ScanBuild" => {
                     #[allow(non_camel_case_types)]
-                    struct ScanSvc<T: Builder>(pub Arc<T>);
-                    impl<T: Builder> tonic::server::UnaryService<super::ScanRequest> for ScanSvc<T> {
-                        type Response = super::ScanResponse;
+                    struct ScanBuildSvc<T: Builder>(pub Arc<T>);
+                    impl<T: Builder> tonic::server::UnaryService<super::ScanBuildRequest> for ScanBuildSvc<T> {
+                        type Response = super::ScanBuildResponse;
                         type Future = BoxFuture<tonic::Response<Self::Response>, tonic::Status>;
                         fn call(
                             &mut self,
-                            request: tonic::Request<super::ScanRequest>,
+                            request: tonic::Request<super::ScanBuildRequest>,
                         ) -> Self::Future {
                             let inner = self.0.clone();
-                            let fut = async move { (*inner).scan(request).await };
+                            let fut = async move { (*inner).scan_build(request).await };
                             Box::pin(fut)
                         }
                     }
@@ -364,7 +436,73 @@ pub mod builder_server {
                     let inner = self.inner.clone();
                     let fut = async move {
                         let inner = inner.0;
-                        let method = ScanSvc(inner);
+                        let method = ScanBuildSvc(inner);
+                        let codec = tonic::codec::ProstCodec::default();
+                        let mut grpc = tonic::server::Grpc::new(codec).apply_compression_config(
+                            accept_compression_encodings,
+                            send_compression_encodings,
+                        );
+                        let res = grpc.unary(method, req).await;
+                        Ok(res)
+                    };
+                    Box::pin(fut)
+                }
+                "/build.Builder/DeleteBuildCache" => {
+                    #[allow(non_camel_case_types)]
+                    struct DeleteBuildCacheSvc<T: Builder>(pub Arc<T>);
+                    impl<T: Builder> tonic::server::UnaryService<super::DeleteBuildCacheRequest>
+                        for DeleteBuildCacheSvc<T>
+                    {
+                        type Response = super::DeleteBuildCacheResponse;
+                        type Future = BoxFuture<tonic::Response<Self::Response>, tonic::Status>;
+                        fn call(
+                            &mut self,
+                            request: tonic::Request<super::DeleteBuildCacheRequest>,
+                        ) -> Self::Future {
+                            let inner = self.0.clone();
+                            let fut = async move { (*inner).delete_build_cache(request).await };
+                            Box::pin(fut)
+                        }
+                    }
+                    let accept_compression_encodings = self.accept_compression_encodings;
+                    let send_compression_encodings = self.send_compression_encodings;
+                    let inner = self.inner.clone();
+                    let fut = async move {
+                        let inner = inner.0;
+                        let method = DeleteBuildCacheSvc(inner);
+                        let codec = tonic::codec::ProstCodec::default();
+                        let mut grpc = tonic::server::Grpc::new(codec).apply_compression_config(
+                            accept_compression_encodings,
+                            send_compression_encodings,
+                        );
+                        let res = grpc.unary(method, req).await;
+                        Ok(res)
+                    };
+                    Box::pin(fut)
+                }
+                "/build.Builder/ScanBuildCache" => {
+                    #[allow(non_camel_case_types)]
+                    struct ScanBuildCacheSvc<T: Builder>(pub Arc<T>);
+                    impl<T: Builder> tonic::server::UnaryService<super::ScanBuildCacheRequest>
+                        for ScanBuildCacheSvc<T>
+                    {
+                        type Response = super::ScanBuildCacheResponse;
+                        type Future = BoxFuture<tonic::Response<Self::Response>, tonic::Status>;
+                        fn call(
+                            &mut self,
+                            request: tonic::Request<super::ScanBuildCacheRequest>,
+                        ) -> Self::Future {
+                            let inner = self.0.clone();
+                            let fut = async move { (*inner).scan_build_cache(request).await };
+                            Box::pin(fut)
+                        }
+                    }
+                    let accept_compression_encodings = self.accept_compression_encodings;
+                    let send_compression_encodings = self.send_compression_encodings;
+                    let inner = self.inner.clone();
+                    let fut = async move {
+                        let inner = inner.0;
+                        let method = ScanBuildCacheSvc(inner);
                         let codec = tonic::codec::ProstCodec::default();
                         let mut grpc = tonic::server::Grpc::new(codec).apply_compression_config(
                             accept_compression_encodings,
