@@ -2,14 +2,17 @@ pub mod api;
 mod app;
 mod bootstrap;
 mod build;
+mod catalog;
 mod config;
 mod constants;
 mod errors;
 pub mod grpc;
+mod logging;
 mod manifest;
 mod namespace;
 mod project;
 mod register;
+mod resource;
 mod service;
 mod time;
 mod utils;
@@ -17,13 +20,16 @@ mod utils;
 pub use app::*;
 pub use bootstrap::*;
 pub use build::*;
+pub use catalog::*;
 pub use config::*;
 pub use constants::*;
 pub use errors::*;
+pub use logging::*;
 pub use manifest::*;
 pub use namespace::*;
 pub use project::*;
 pub use register::*;
+pub use resource::*;
 pub use service::*;
 pub use time::*;
 pub use utils::*;
